@@ -636,6 +636,12 @@ int mutt_index_menu (void)
 	  mutt_message _("New mail in this mailbox.");
 	  if (option (OPTBEEPNEW))
 	    beep ();
+	  if (NewMailCmd)
+	  {
+	    char cmd[LONG_STRING];
+	    menu_status_line(cmd, sizeof(cmd), menu, NONULL(NewMailCmd));
+	    mutt_system(cmd);
+	  }
 	} else if (check == MUTT_FLAGS)
 	  mutt_message _("Mailbox was externally modified.");
 
@@ -662,6 +668,12 @@ int mutt_index_menu (void)
          menu->redraw |= REDRAW_STATUS;
          if (option (OPTBEEPNEW))
            beep();
+	 if (NewMailCmd)
+	 {
+	   char cmd[LONG_STRING];
+	   menu_status_line(cmd, sizeof(cmd), menu, NONULL(NewMailCmd));
+	   mutt_system(cmd);
+	 }
        }
      }
      else
@@ -914,6 +926,12 @@ int mutt_index_menu (void)
 	menu->redraw = REDRAW_FULL;
 	break;
 
+      case OP_ERROR_HISTORY:
+
+	mutt_error_history_display ();
+	menu->redraw = REDRAW_FULL;
+	break;
+
       case OP_MAIN_SHOW_LIMIT:
 	CHECK_IN_MAILBOX;
 	if (!Context->pattern)
@@ -1213,6 +1231,11 @@ int mutt_index_menu (void)
 #endif
 	else
 	{
+          if (option (OPTCHANGEFOLDERNEXT) && Context && Context->path)
+          {
+            strfcpy (buf, Context->path, sizeof (buf));
+            mutt_pretty_mailbox (buf, sizeof (buf));
+          }
 	  mutt_buffy (buf, sizeof (buf));
 
           if (mutt_enter_fname (cp, buf, sizeof (buf), 1) == -1)
