@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
+ * Copyright (C) 1996-1999 Brandon Long <blong@fiction.net>
  * Copyright (C) 1999-2000,2005 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
@@ -37,13 +37,12 @@ typedef struct imap_header_data
   unsigned int parsed : 1;
 
   unsigned int uid;	/* 32-bit Message UID */
+  unsigned int msn;     /* Message Sequence Number */
   LIST *keywords;
 } IMAP_HEADER_DATA;
 
 typedef struct
 {
-  unsigned int sid;
-
   IMAP_HEADER_DATA* data;
 
   time_t received;
