@@ -902,8 +902,12 @@ void mutt_pretty_mailbox (char *s, size_t buflen)
 
 void mutt_pretty_size (char *s, size_t len, LOFF_T n)
 {
-  if (n < 1000)
-    snprintf (s, len, "%d", (int)n);
+  if (n == 0)
+    strfcpy (s, "0K", len);
+  /* Change in format released in 1.10.0, but reverted after feedback:
+   * if (n < 1000)
+   *  snprintf (s, len, "%d", (int)n);
+   */
   else if (n < 10189) /* 0.1K - 9.9K */
     snprintf (s, len, "%3.1fK", (n < 103) ? 0.1 : n / 1024.0);
   else if (n < 1023949) /* 10K - 999K */
